/*
*  WhatsNew utility
*  
*  source/common.c
*  snapshot 20040325
*  
*  (c) 27.3.2002, 24.12.2003 Jaroslav Beran berk@volny.cz http://www.volny.cz/berk
*  License GNU/GPL2  
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#ifdef __linux__
#include <unistd.h>
#else
#include <io.h>
#endif
#include "common.h"


void strip_sp(char *s)
{
        int first = 0,
            last = 0,
            delka,
            n;


        while(isspace(s[first]))
                first++;

        last = strlen(s) - 1;

        while((isspace(s[last]) || (s[last] == '\n')) && (last > first))
                last--;

        delka = (last + 1) - first;

        for(n = 0; n < delka; n++)
                s[n] = s[first + n];

        s[n]='\0';

        return;
}


int add_entry(char ***arr, int *size, char *entry)
{
        char **new_arr;
        int n;


        if((new_arr = (char **) malloc((*size + 1) * sizeof(char *))) == NULL){
                printf("add_entry: Neni dostatek pameti\n");
                return -1;
        }

        for(n = 0; n < *size; n++)
                *(new_arr + n) = *(*arr + n);

        if(*size > 0) 
                free((void *) *arr);

        *(new_arr + *size) = entry;
        *arr = new_arr;
        (*size)++;

        return 0;
}


int arr_contain(char **arr, int n, char *s)
{
        int i;

        for (i = 0; i < n; i++){
                if (!strcmp(arr[i], s))
                        return 0;
        }

        return 1;
}


void free_array(char ***arr, int max)
{
        int n;

        if (*arr == NULL)
                return;
        
        for(n = 0; n < max; n++)
                if (*(*arr + n) != NULL)                                
                        free((void *) *(*arr + n));

        free((void *) *arr);
        return;
}


extern char *dup_str(char *s)
{
        char *p;


        if (s == NULL)
                return NULL;

        if ((p = (char *) malloc((strlen(s) + 1) * sizeof(char))) == NULL)
                return NULL;

        strcpy(p, s);
        return p;
}


int add_str(char **where, char *what)
{
        char *p;


        if(what == NULL)
                return 0;

        if(*where == NULL){

                if ((*where = (char *) malloc((strlen(what) + 1) * sizeof(char))) == NULL){
                        printf("add_str: Neni dostatek pameti\n");
                        return -1;
                }
                strcpy(*where, what);

        } else {

                if ((p = (char *) malloc((strlen(*where) + strlen(what) + 1) * sizeof(char))) == NULL){
                        printf("add_str: Neni dostatek pameti\n");
                        return -1;
                }

                *p = '\0';
                strcat(p, *where);
                strcat(p, what);
                free((void *) *where);
                *where = p;
        }

        return 0;
}


int add_buf_str(char *where, char *what, int max)
{
        if((strlen(where) + strlen(what)) >= (max - 1))
                what[max - 1 - strlen(where)] = '\0';

        strcat(where, what);

        return(max - 1 - strlen(where));
}


char *build_path(const char *dir, const char *fn, const char *ext)
{
        char *s;


        if((s = (char *) malloc( \
                (strlen(dir) + strlen(DIVIDER) + strlen(fn) + strlen(ext) + 1) \
                * sizeof(char))) == NULL){
                        fprintf(stderr, "build_path: cant allocate memory");
        } else {
                strcpy(s, dir);
                if(strcmp(s + strlen(s) - 1, DIVIDER)) 
                        strcat(s, DIVIDER);
                strcat(s, fn);
                strcat(s, ext);
        }

        return s;
}


FILE *my_fopen(char *path, char *filename, char *mode)
{
        char *pom;
        FILE *f;


        if ((pom = build_path(path, filename, "\0")) == NULL)
                return NULL;

        f = fopen(pom, mode);

        free((void *) pom);
        return f;
}


int my_touch(char *filename)
{
        FILE *f;


        if (filename == NULL)
                return -1;

        if (access(filename, F_OK) != 0){

                if ((f = fopen(filename, "w")) == NULL)
                        return -1;

                fclose(f);
        }

        return 0;
}


int my_touch2(char *path, char *filename)
{
        char *pom;
        int ret;


        if ((path == NULL) || (filename == NULL))
                return -1;

        if ((pom = build_path(path, filename, "\0")) == NULL)
                return -1;

        ret = my_touch(pom);
        free((void *) pom);

        return ret;

}
