/*
*  WhatsNew utility
*  
*  source/main.c
*  snapshot 20040326
*  
*  (c) 27.3.2002, 24.12.2003 Jaroslav Beran berk@volny.cz http://www.volny.cz/berk
*  License GNU/GPL2  
*/


#define VERSION "WhatsNew snapshot 20040326"


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <time.h>
#ifdef __linux__
#include <unistd.h>
#else
#include <io.h>
#endif
#include "common.h"
#include "rcfiles.h"
#include "logger.h"
#include "pidfile.h"
#include "reporter.h"
#include "statistics.h"


#ifndef CONF_FILE
        #define CONF_FILE "whatsnew.conf"
#endif

#define ENTRYLEN 512
#define TMP_TABLE1 "~table_1.tmp"


struct s_init_data {
        char db_path[ENTRYLEN],
             table_name[ENTRYLEN],
             journal[ENTRYLEN],
             log_file[ENTRYLEN],
             report_file[ENTRYLEN],
             report_tmp_file[ENTRYLEN],
             pid_file[ENTRYLEN];
        char **zdroje;
        int  zdroje_pocet;
};


void print_help(char *name);
int run(char *conf_file);
int clean_db(char *conf_file);
int init(char *conf_file, struct s_init_data *data);
void cleanup(struct s_init_data *data);
int list_dir(const char *path, char *section, struct h_entry *entry);
int update_db(char *section, char *db_path, char *table, char *tmp_table, char *journal, struct s_stat *st);
int update_section(char *section, char *db_path, char *table, char *tmp_table);
int str_chk(char *s);
char *get_rel_path(char *abs_path, char *source);
int rename_file(char *target, char *source, char *db_path);


int main(int argc, char *argv[])
{
        int count;
        char *argument = NULL;


        count = argc - 1;

        while (count > 0){
	        if(argv[count][0] == '-'){

                        switch(argv[count][1]){
                                case 'c' :
                                        if (argument == NULL)
                                                return clean_db(CONF_FILE);
                                        else
                                                return clean_db(argument);

		                        return 0;
	                        case 'h' :
                                        print_help(argv[0]);
		                        return 0;
                                case 'r' :
                                        if (argument == NULL)
                                                return run(CONF_FILE);
                                        else
                                                return run(argument);
                                case 'v' :
                                        printf("%s\n", VERSION);
                                        return 0;
	                        default  :
                                        print_help(argv[0]);
		                        return 0;
	                }

                        argument = NULL;

	        } else
                        argument = argv[count];
	        
                count--;
        }

        print_help(argv[0]);
        return -1;
}


void print_help(char *name)
{
        printf("%s - utilitka pro zjistovani novych souboru na disku\n", VERSION);
        printf("(c)2002 Jaroslav Beran - berk@volny.cz http://www.volny.cz/berk\n");
        printf("Licence GNU/GPL2 (http://www.gnu.cz)\n\n");

        printf("Pouziti:\n");
        printf("%s [options]\n\n", name);

        printf("Options:\n");
        printf(" -c\t\t(cleanup) provede vycisteni databaze\n");
        printf(" -h\t\t(help) vypise tento help\n");
        printf(" -r [conf_file]\t(run) spusti test (defaultne %s)\n", CONF_FILE);
        printf(" -v\t\t(version) verze programu\n");

        return;
}


int run(char *conf_file)
{
        int  ret,
             n;
        struct s_init_data data;
        struct h_entry table_tmp;
        struct s_stat local_st, 
                      global_st;


        if((ret = init(conf_file, &data)) != 0)
                return ret;

        st_start(&global_st);

        for (n = 0; n < data.zdroje_pocet; n++){
                rep_writeln("[%s]", *(data.zdroje + n));
                printf("\n[%s]\n", *(data.zdroje + n));

                st_start(&local_st);

                if (open_table(&table_tmp, data.db_path, TMP_TABLE1, "w") == NULL){
                        log_message(LM_FATAL, "Nemohu otevrit soubor %s", TMP_TABLE1);
                        cleanup(&data);
                        return -1;
                }

                printf("checking..\n");
                if ((ret = list_dir(*(data.zdroje + n), *(data.zdroje + n), &table_tmp)) < 0){
                        log_message(LM_FATAL, "Pri prochazeni zdrojem %s nastala chyba", *(data.zdroje + n));                        
                        close_table(&table_tmp);
                        cleanup(&data);
                        return -1;
                }

                close_table(&table_tmp);

                if (ret != 1){
                        printf("updating..\n");
                        if (update_db(*(data.zdroje + n), data.db_path, data.table_name, TMP_TABLE1, data.journal, &local_st)){
                                log_message(LM_FATAL, "Nepodarilo se updatovat databazi zdroje %s", *(data.zdroje + n));
                                cleanup(&data);
                                return -1;
                        }
                } else {
                        rep_writeln("(!) n/a");
                        printf("n/a\n");
                }

                rep_writeln("");
                st_stop(&local_st);
                st_add(&global_st, &local_st);
                st_print(rep_getfd(), &local_st);
                rep_writeln("\n");
        }

        printf("\nDone\n\n");
        log_message(LM_NOTICE, "Done");

        st_stop(&global_st);
        st_print(stdout, &global_st);

        cleanup(&data);

        return 0;
}


int clean_db(char *conf_file)
{
        int ret, i;
        struct s_init_data data;
        struct h_entry table,
                       table_tmp;
        char buff1[MAX_DB_LINE];
        char **deleted,
             *p;
        int deleted_count = 0;


        if ((ret = init(conf_file, &data)) != 0)
                return ret;

        if (my_touch2(data.db_path, data.table_name)){
                log_message(LM_WARN, "update_db: can't create file %s", data.table_name);
                cleanup(&data);
                return -1;
        }

        if (open_table(&table, data.db_path, data.table_name, "r") == NULL){
                log_message(LM_WARN, "update_db: can't open file %s", data.table_name);
                cleanup(&data);
                return -1;
        }

        if (open_table(&table_tmp, data.db_path, TMP_TABLE1, "w") == NULL){
                log_message(LM_FATAL, "Nemohu otevrit soubor %s", TMP_TABLE1);
                close_table(&table);
                cleanup(&data);
                return -1;
        }

        printf("\nCleaning..\n\n");
        rep_writeln("Cleaning..\n");

        while(!get_next_rec(&table, buff1, MAX_DB_LINE)){

                if (!arr_contain(data.zdroje, data.zdroje_pocet, table.source)){
                        transfer_fields(&table_tmp, &table);
                        if (put_rec(&table_tmp))
                                log_message(LM_ERROR, "update_table: chyba zpisu do %s", TMP_TABLE1);
                } else {
                        if (arr_contain(deleted, deleted_count, table.source) &&
                           ((p = dup_str(table.source)) != NULL))
                                add_entry(&deleted, &deleted_count, p);
                }
        }

        close_table(&table_tmp);
        close_table(&table);

        if (deleted_count > 0){
                printf("Odstranene sekce:\n\n");
                rep_writeln("Odstranene sekce:\n");
                for (i = 0; i < deleted_count; i++){
                        printf("%s\n", deleted[i]);
                        rep_writeln("%s", deleted[i]);
                }
                free_array(&deleted, deleted_count);
        } else {
                printf("Databaze je aktualni\n");
                rep_writeln("Databaze je aktualni");
        }

        ret = rename_file(data.table_name, TMP_TABLE1, data.db_path);

        rep_writeln("");
        printf("\nDone\n\n");
        log_message(LM_NOTICE, "Done");
        cleanup(&data);
        return ret;
}


int init(char *conf_file, struct s_init_data *data)
{
        FILE *f;


        printf("Using\t\t%s\n", conf_file);

        if ((f = fopen(conf_file, "r")) == NULL){
                printf("run: can't open file %s\n", conf_file);
                return -1;
        }

        /* pid_file */
        if (fget_buf_item(data->pid_file, ENTRYLEN, "GENERAL", "PID_FILE", f)){
                printf("run: can't find entry PID_FILE in GENERAL section\n");
                return -1;
        }
        printf("PID_FILE\t%s\n", data->pid_file);
        fseek(f, 0, SEEK_SET);

        if (create_pidfile(data->pid_file))
                return -1;

        /* db_path */
        if (fget_buf_item(data->db_path, ENTRYLEN, "GENERAL", "DB_PATH", f)){
                printf("run: can't find entry DB_PATH in GENERAL section\n");
                erase_pidfile(data->pid_file);
                return -1;
        }
        printf("DB_PATH\t\t%s\n", data->db_path);
        fseek(f, 0, SEEK_SET);

        /* table_name */
        if (fget_buf_item(data->table_name, ENTRYLEN, "GENERAL", "TABLE_NAME", f)){
                printf("run: can't find entry TABLE_NAME in GENERAL section\n");
                erase_pidfile(data->pid_file);
                return -1;
        }
        printf("TABLE_NAME\t%s\n", data->table_name);
        fseek(f, 0, SEEK_SET);

        /* journal */
        if (fget_buf_item(data->journal, ENTRYLEN, "GENERAL", "JOURNAL", f)){
                printf("run: can't find entry JOURNAL in GENERAL section\n");
                erase_pidfile(data->pid_file);
                return -1;
        }
        printf("JOURNAL\t\t%s\n", data->journal);
        fseek(f, 0, SEEK_SET);

        /* log_file */
        if (fget_buf_item(data->log_file, ENTRYLEN, "GENERAL", "LOG_FILE", f)){
                printf("run: can't find entry LOG_FILE in GENERAL section\n");
                erase_pidfile(data->pid_file);
                return -1;
        }
        printf("LOG_FILE\t%s\n", data->log_file);
        fseek(f, 0, SEEK_SET);

        /* report_file */
        if (fget_buf_item(data->report_file, ENTRYLEN, "GENERAL", "REPORT_FILE", f)){
                printf("run: can't find entry REPORT_FILE in GENERAL section\n");
                erase_pidfile(data->pid_file);
                return -1;
        }
        printf("REPORT_FILE\t%s\n", data->report_file);
        fseek(f, 0, SEEK_SET);

        /* report_tmp_file */
        if (fget_buf_item(data->report_tmp_file, ENTRYLEN, "GENERAL", "REPORT_TMP_FILE", f)){
                printf("run: can't find entry REPORT_TMP_FILE in GENERAL section\n");
                erase_pidfile(data->pid_file);
                return -1;
        }
        printf("REPORT_TMP_FILE\t%s\n", data->report_tmp_file);
        fclose(f);

        set_log_file(data->log_file);
        log_message(LM_NOTICE, "Starting with %s", conf_file);


        if (rep_init(data->report_file, data->report_tmp_file)){
                log_message(LM_FATAL, "Nemohu inicializovat reporter (%s)", data->report_tmp_file);
                erase_pidfile(data->pid_file);
                return -1;
        }

        if ((data->zdroje = sget_strs(&(data->zdroje_pocet), conf_file, "PATHS")) == NULL){
                log_message(LM_FATAL, "Nepodarilo se nacist polozky ze sekce [PATHS]");
                rep_finish();
                erase_pidfile(data->pid_file);
                return -1;
        }

        return 0;
}


void cleanup(struct s_init_data *data)
{
        free_array(&(data->zdroje), data->zdroje_pocet);
        rep_finish();
        erase_pidfile(data->pid_file);

        return;
}


int list_dir(const char *path, char *section, struct h_entry *entry)
{
        char *p_t;
        time_t cas = 0;
        DIR *p_currdir;
        struct dirent *p_currfile;
        struct stat atributy;


        if ((p_currdir = opendir(path)) == NULL){
                if (!strcmp(section, path))
                        return 1;
                else {
                        /* rep_writeln("NEPRISTUPNY ADRESAR %s <DIR>", path); */
                        return 0;
                }
        }

        while((p_currfile = readdir(p_currdir)) != NULL){
                if (!strcmp(p_currfile->d_name, ".")) continue;
                if (!strcmp(p_currfile->d_name, "..")) continue;

                if((p_t = build_path(path, p_currfile->d_name, "\0")) == NULL){
                        log_message(LM_FATAL, "list_dir: neni dostatek pameti");
                        closedir(p_currdir);
                        return -1;
                }
                str_chk(p_t);

                stat(p_t, &atributy);

                entry->source = section;
                entry->file = get_rel_path(p_t, section);
                entry->size = atributy.st_size;
                entry->modified = atributy.st_mtime;
                entry->explored_created = time(&cas);
                entry->explored_deleted = 0;
                entry->mode = atributy.st_mode;

                if (put_rec(entry))
                        log_message(LM_ERROR, "list_dir: nepodarilo se zapsat soubor %s", p_t);

                if(S_ISDIR(atributy.st_mode))
                        if(list_dir(p_t, section, entry)){
                                free((void *) p_t);
                                closedir(p_currdir);
                                return -1;
                        }

                free((void *) p_t);
        }

        closedir(p_currdir);
        return 0;
}


int update_db(char *section, char *db_path, char *table, char *tmp_table, char *journal, struct s_stat *st)
{
        char buff1[MAX_DB_LINE],
             buff2[MAX_DB_LINE];
        time_t cas = 0;
        struct h_entry table_e,
                       tmp_e,
                       journal_e;


        if (my_touch2(db_path, table)){
                log_message(LM_WARN, "update_db: can't create file %s", table);
                return -1;        
        }

        if (open_table(&table_e, db_path, table, "r") == NULL){
                log_message(LM_WARN, "update_db: can't open file %s", table);
                return -1;
        }

        if (open_table(&tmp_e, db_path, tmp_table, "r") == NULL){
                log_message(LM_WARN, "update_db: can't open file %s", tmp_table);
                close_table(&table_e);
                return -1;
        }

        if (open_table(&journal_e, db_path, journal, "a") == NULL){
                log_message(LM_WARN, "update_db: can't open file %s", journal);
                close_table(&table_e);
                close_table(&tmp_e);
                return -1;
        }

        set_table_filter(&table_e, section);
        seek_top(&table_e);

        while(!get_next_rec(&tmp_e, buff1, MAX_DB_LINE)){

                if (find_rec(&table_e, buff2, MAX_DB_LINE, tmp_e.source, tmp_e.file)){
                        rep_write(&tmp_e, REP_NEW);
                        st->new_files++;
                } else
                        if ((table_e.modified != tmp_e.modified) || (table_e.size != tmp_e.size)){
                                rep_write(&tmp_e, REP_MODIFIED);
                                st->modified_files++;
                        }

                st->all_files++;
        }

        seek_top(&table_e);

        while(!get_next_rec(&table_e, buff1, MAX_DB_LINE)){

                if ((!strcmp(table_e.source, section)) && 
                     find_rec(&tmp_e, buff2, MAX_DB_LINE, table_e.source, table_e.file)) {

                        rep_write(&table_e, REP_DELETED);
                        st->deleted_files++;
                        table_e.explored_deleted = time(&cas);
                        transfer_fields(&journal_e, &table_e);
                        if (put_rec(&journal_e))
                                log_message(LM_WARN, "update_db: chyba zpisu do %s", journal);

                }
        }

        close_table(&table_e);
        close_table(&tmp_e);
        close_table(&journal_e);

        if (update_section(section, db_path, table, tmp_table))
                return -1;

        return 0;
}


int update_section(char *section, char *db_path, char *table, char *tmp_table)
{
        char buff1[MAX_DB_LINE];
        struct h_entry table_e,
                       tmp_e;


        if (open_table(&table_e, db_path, table, "r") == NULL){
                log_message(LM_ERROR, "update_table: can't open file %s", table);
                return -1;
        }

        if (open_table(&tmp_e, db_path, tmp_table, "a") == NULL){
                log_message(LM_ERROR, "update_table: can't open file %s", tmp_table);
                close_table(&table_e);
                return -1;
        }

        while(!get_next_rec(&table_e, buff1, MAX_DB_LINE)){

                if (strcmp(table_e.source, section)){
                        transfer_fields(&tmp_e, &table_e);
                        if (put_rec(&tmp_e))
                                log_message(LM_ERROR, "update_table: chyba zpisu do %s", tmp_table);
                }
        }

        close_table(&table_e);
        close_table(&tmp_e);

        return rename_file(table, tmp_table, db_path);
}


int str_chk(char *s){
        int ret = 0;

        if (s == NULL)
                return ret;

        while(*s != '\0'){
                if(*s == 0x1A){
                        ret = 0x1A;
                        *s = '!';
                }

                s++;
        }

        if (ret)
                log_message(LM_WARN, "str_chk: soubor %s obsahuje v nazvu znak %x", s, ret);

        return ret;
}


char *get_rel_path(char *abs_path, char *source)
{
        if (abs_path == NULL)
                return NULL;

        if (source == NULL)
                return abs_path;

        if (*(abs_path + strlen(source)) == DIVIDER[0])
                return(abs_path + strlen(source) + 1);
        else
                return(abs_path + strlen(source));
}


int rename_file(char *target, char *source, char *db_path)
{
        char *p_table,
             *p_tmp;
        int ret;


        if ((p_table = build_path(db_path, target, "\0")) == NULL){
                log_message(LM_ERROR, "update_table: can't build path");
                return -1;
        }

        if ((p_tmp = build_path(db_path, source, "\0")) == NULL){
                log_message(LM_ERROR, "update_table: can't build path");
                free((void *) p_table);
                return -1;
        }

        unlink(p_table);
        if ((ret = rename(p_tmp, p_table)) != 0)
                log_message(LM_ERROR, "update_table: cant rename %s to %s", p_tmp, p_table);

        free((void *) p_table);
        free((void *) p_tmp);

        return ret;
}
