/*
*  WhatsNew utility
*  
*  source/mydblib.c
*  snapshot 20040131
*  
*  (c) 27.3.2002, 24.12.2003 Jaroslav Beran berk@volny.cz http://www.volny.cz/berk
*  License GNU/GPL2  
*/


#include <string.h>
#include "slash.h"
#include "common.h"
#include "mydblib.h"


FILE *open_table(struct h_entry *entry, char *db_path, char *table, char *mode)
{
        entry->f = my_fopen(db_path, table, mode);
        entry->last_pos = 0;
        entry->filter_origin = -1;
        entry->filter_end = -1;

        return entry->f;
}


int close_table(struct h_entry *entry)
{
        int ret;


        if (entry->f == NULL)
                return -1;

        ret = fclose(entry->f);
        entry->last_pos = 0;
        entry->filter_origin = -1;
        entry->filter_end = -1;

        return ret;
}


int put_rec(struct h_entry *entry)
{
        char source[MAX_ENTRY_LEN],
             file[MAX_ENTRY_LEN];


        if (entry->f == NULL)
                return -1;

        if (strlen(entry->source) > (MAX_ENTRY_LEN - 1)){
                fprintf(stderr, "write_mydb: MAX_ENTRY_LEN is small\n");
                return -1; 
        }
        strcpy(source, entry->source);

        if (slash_str(DB_DIVIDER, source, MAX_ENTRY_LEN) == 1)
                fprintf(stderr, "write_mydb: MAX_ENTRY_LEN is small\n"); 

        if (strlen(entry->file) > (MAX_ENTRY_LEN - 1)){
                fprintf(stderr, "write_mydb: MAX_ENTRY_LEN is small\n");
                return -1; 
        }
        strcpy(file, entry->file);

        if (slash_str(DB_DIVIDER, file, MAX_ENTRY_LEN) == 1)
                fprintf(stderr, "write_mydb: MAX_ENTRY_LEN is small\n"); 

        fprintf(entry->f, "%s%c" \
                          "%s%c" \
                          "%li%c" \
                          "%li%c" \
                          "%li%c" \
                          "%li%c" \
                          "%hu%c" \
                          "\n", \
                          source, DB_DIVIDER, \
                          file, DB_DIVIDER, \
                          entry->size, DB_DIVIDER, \
                          entry->modified, DB_DIVIDER, \
                          entry->explored_created, DB_DIVIDER, \
                          entry->explored_deleted, DB_DIVIDER, \
                          entry->mode, DB_DIVIDER);

        return 0;
}


int get_next_rec(struct h_entry *entry, char *buff, int max)
{
        char *pbuff;
        int m,
            n = 0,
            separator = 0;


        if ((entry->f == NULL) || (buff == NULL))
                return -1;

        if (out_of_filter(entry))
                return -1;

        if (fgets(buff, max, entry->f) == NULL)
                return -1;

        pbuff = buff;
        while(*pbuff != '\0'){

                if ((*pbuff == DB_DIVIDER) && !separator){
                        *pbuff = '\0';
                        n++;
                }

                if ((*pbuff == SLASHCHAR) && !separator)
                        separator = 1;
                else
                        separator = 0;

                pbuff++;
        }

        entry->source = NULL;
        entry->file = NULL;
        entry->size = 0;
        entry->modified = 0;
        entry->explored_created = 0;
        entry->explored_deleted = 0;
        entry->mode = 0;

        pbuff = buff;
        for(m = 0; m < n; m++){
                
                switch(m) {

                        case 0 :
                                entry->source = pbuff;
                                break;

                        case 1 :
                                entry->file = pbuff;
                                break;

                        case 2 :
                                sscanf(pbuff, "%li", &(entry->size));
                                break;

                        case 3 :
                                sscanf(pbuff, "%li", &(entry->modified));
                                break;

                        case 4 :
                                sscanf(pbuff, "%li", &(entry->explored_created));
                                break;

                        case 5 :
                                sscanf(pbuff, "%li", &(entry->explored_deleted));
                                break;

                        case 6 :
                                sscanf(pbuff, "%hu", &(entry->mode));
                                break;
                }

                pbuff += strlen(pbuff) + 1;
        }

        unslash_str(entry->source);
        unslash_str(entry->file);

        return 0;
}


int find_rec(struct h_entry *entry, char *buff, int max, char *source, char *file)
{
        if (entry->f == NULL)
                return -1;
        
        fseek(entry->f, entry->last_pos, SEEK_SET);
        if ((!out_of_filter(entry)) &&
            (!get_next_rec(entry, buff, max)) &&
            (!strcmp(entry->file, file)) && 
            (!strcmp(entry->source, source))){
                entry->last_pos = ftell(entry->f);
                return 0;
        }

        seek_top(entry);

        while((!out_of_filter(entry)) && (!get_next_rec(entry, buff, max))){
                if ((!strcmp(entry->file, file)) && (!strcmp(entry->source, source))){
                        entry->last_pos = ftell(entry->f);
                        return 0;
                }
        }

        return 1;
}


int set_table_filter(struct h_entry *entry, char *source)
{
        char buff[MAX_DB_LINE];
        long prev_pos = 0,
             origin = -1;


        if (entry->f == NULL)
                return -1;

        entry->last_pos = 0;
        entry->filter_origin = -1;
        entry->filter_end = -1;

        if (source == NULL)
                return 0;

        fseek(entry->f, 0, SEEK_SET);

        while((origin == -1) && !get_next_rec(entry, buff, MAX_DB_LINE)){

                if (!strcmp(entry->source, source))
                        origin = prev_pos;

                prev_pos = ftell(entry->f);
        }

        if (origin == -1){
                entry->filter_origin = ftell(entry->f);
                entry->filter_end = ftell(entry->f);

                return 0;
        }

        fseek(entry->f, origin, SEEK_SET);

        while(!get_next_rec(entry, buff, MAX_DB_LINE) && !strcmp(entry->source, source));

        entry->filter_origin = origin;
        entry->filter_end = ftell(entry->f);

        return 0;
}


int out_of_filter(struct h_entry *entry)
{
        long pos;


        if (entry->f == NULL)
                return -1;

        if(entry->filter_origin == -1)
                return 0;

        pos = ftell(entry->f);

        if (pos >= entry->filter_end)
                return 1;

        if (pos < entry->filter_origin)
                return 1;

        return 0;
}


void seek_top(struct h_entry *entry)
{
        if (entry->filter_origin == -1)
                fseek(entry->f, 0, SEEK_SET);
        else
                fseek(entry->f, entry->filter_origin, SEEK_SET);

        return;
}


void transfer_fields(struct h_entry *target, struct h_entry *source)
{
        target->source = source->source;
        target->file = source->file;
        target->size = source->size;
        target->modified = source->modified;
        target->explored_created = source->explored_created;
        target->explored_deleted = source->explored_deleted;
        target->mode = source->mode;

        return;
}
