/*
*  RcFiles verze 0.3 cz
*  
*  source/rcfiles.c
*  snapshot 20031229
*  
*  (c) 27.3.2002 Jaroslav Beran berk@volny.cz http://www.volny.cz/berk
*  License GNU/GPL2  
*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "common.h"
#include "rcfiles.h"


#define SECTIONBUF 50
#define LINEBUF 256
#define BEGINSECTION '['
#define ENDSECTION ']'
#define COMMENT '#'
#define ASSIGN '='
#define LINECONTINUE '\\'


/* is_comment - vraci Ano=1, Ne=0 */
int is_comment(const char *s);
/* is_empty - vraci Ano=1, Ne=0 */
int is_empty(const char *s);
/* is_section - Ano=1, Ne=0, v sekce vraci jeji jmeno (max 49 znaku) */
int is_section(const char *s, char *sekce, unsigned max);
void break_line(char *line, char **pname, char **pvalue, char assignchar);


char **sget_sections(int *pocet, const char *rcfile)
{
        FILE *fr;
        char **ret;


        if((fr = fopen(rcfile, "r")) == NULL){
                printf("sget_sections: Nemohu otevrit soubor %s\n", rcfile);
                return NULL;
        }

        ret = fget_sections(pocet, fr);
        fclose(fr);

        return ret;
}


char **fget_sections(int *pocet, FILE *f)
{
        char buff[LINEBUF],
             sekce[SECTIONBUF],
             *pom,
             **arr_ret = NULL;


        *pocet = 0;

        while(fgets(buff, LINEBUF - 1, f) != NULL){
                if(is_comment(buff)) continue;
                if(is_empty(buff)) continue;
                if(is_section(buff, sekce, SECTIONBUF)){

                        if((pom = (char *) malloc((strlen(sekce) + 1) * sizeof(char))) == NULL){
                                printf("fget_sections: Neni dostatek pameti\n");
                                free_array(&arr_ret, *pocet);
                                return NULL;
                        }

                        strcpy(pom, sekce);

                        if(add_entry(&arr_ret, pocet, pom)){
                                printf("fget_sections: Neni dostatek pameti\n");
                                free_array(&arr_ret, *pocet);
                                return NULL;
                        }
                }
        }

        return arr_ret;
}


char **sget_strs(int *pocet, const char *rcfile, const char *sekce)
{
        FILE *fr;
        char **ret;


        if((fr = fopen(rcfile, "r")) == NULL){
                printf("sget_strs: Nemohu otevrit soubor %s\n", rcfile);
                return NULL;
        }

        ret = fget_strs(pocet, fr, sekce);
        fclose(fr);

        return ret;
}


char **fget_strs(int *pocet, FILE *f, const char *sekce)
{
        char buff[LINEBUF], 
             cur_sekce[SECTIONBUF], 
             *pom,
             **arr_ret = NULL;


        *pocet = 0;

        while(fgets(buff, LINEBUF - 1, f) != NULL){
                if(is_comment(buff)) continue;
                if(is_empty(buff)) continue;
                if(is_section(buff, cur_sekce, SECTIONBUF)) continue;
                if(!strcmp(sekce, cur_sekce)){

                        strip_sp(buff);

                        if((pom = (char *) malloc((strlen(buff) + 1) * sizeof(char))) == NULL){
                                printf("fget_sections: Neni dostatek pameti\n");
                                free_array(&arr_ret, *pocet);
                                return NULL;
                        }

                        strcpy(pom, buff);

                        if(add_entry(&arr_ret, pocet, pom)){
                                printf("fget_sections: Neni dostatek pameti\n");
                                free_array(&arr_ret, *pocet);
                                return NULL;
                        }
                }
        }

        return arr_ret;
}


char *sget_item(const char *sekce, const char *polozka, const char *rcfile)
{
        FILE *fr;
        char *ret;


        if((fr = fopen(rcfile, "r")) == NULL){
                printf("sget_item: Nemohu otevrit soubor %s\n", rcfile);
                return NULL;
        }

        ret = fget_item(sekce, polozka, fr);
        fclose(fr);

        return ret;
}


char *fget_item(const char *sekce, const char *polozka, FILE *f)
{
        char buff[LINEBUF], 
             cur_sekce[SECTIONBUF],
             *ret = NULL, 
             *name,
             *value;


        while(fgets(buff, LINEBUF - 1, f) != NULL){
                if(is_comment(buff)) continue;
                if(is_empty(buff)) continue;
                if(is_section(buff, cur_sekce, SECTIONBUF)) continue;
                if(!strcmp(sekce, cur_sekce)){

                        break_line(buff, &name, &value, ASSIGN);
                        if((name == NULL) || (value == NULL))
                                continue;
                        strip_sp(name);
                        if(strcmp(polozka, name)) continue;

                        strip_sp(value);
                        if(add_str(&ret, value)){
                                if(ret != NULL)
                                        free((void *) ret);
                                return NULL;
                        }

                        while((ret[strlen(ret) - 1] == LINECONTINUE) 
                                && (fgets(buff, LINEBUF - 1, f) != NULL)){

                                ret[strlen(ret) - 1] = '\0';
                                strip_sp(buff);

                                if(add_str(&ret, buff)){
                                        if(ret != NULL)
                                                free((void *) ret);
                                        return NULL;
                                }
                        }
                }
        }

        return ret;
}


int sget_buf_item(char *buff, unsigned max, const char *sekce, const char *polozka, const char *rcfile)
{
        FILE *fr;
        int ret;


        if((fr = fopen(rcfile, "r")) == NULL){
                printf("sget_buf_item: Nemohu otevrit soubor %s\n", rcfile);
                return -1;
        }

        ret = fget_buf_item(buff, max, sekce, polozka, fr);
        fclose(fr);

        return ret;
}


int fget_buf_item(char *buff, unsigned max, const char *sekce, const char *polozka, FILE *f)
{
        char line[LINEBUF],
             cur_sekce[SECTIONBUF],
             *name,
             *value;
        int i,
            ret = 1;

        *buff = '\0';

        while(fgets(line, LINEBUF - 1, f) != NULL){
                if(is_comment(line)) continue;
                if(is_empty(line)) continue;
                if(is_section(line, cur_sekce, SECTIONBUF)) continue;
                if(!strcmp(sekce, cur_sekce)){

                        break_line(line, &name, &value, ASSIGN);
                        if((name == NULL) || (value == NULL))
                                continue;
                        strip_sp(name);
                        if(strcmp(polozka, name)) continue;

                        ret = 0;
                        strip_sp(value);
                        i = add_buf_str(buff, value, max);

                        while((i > 0)
                                && (buff[strlen(buff) - 1] == LINECONTINUE) 
                                && (fgets(line, LINEBUF - 1, f) != NULL)){

                                buff[strlen(buff) - 1] = '\0';
                                strip_sp(line);
                                i = add_buf_str(buff, line, max);
                        }
                }
        }

        return ret;
}


int is_comment(const char *s)
{
        while(isspace(*s))
                s++;

        if(*s == COMMENT) 
                return 1;

        return 0;
}


int is_empty(const char *s)
{
        while(isspace(*s))
                s++;

        if(*s == '\0') 
                return 1;

        return 0;
}


int is_section(const char *s, char *sekce, unsigned max)
{
        while(isspace(*s))
                s++;

        if(*s != BEGINSECTION)
                return 0;
        else 
                s++;

        while((*s != ENDSECTION) && (*s != '\0') && (max > 1)){
                *sekce = *s;
                sekce++;
                s++;
                max--;
        }

        *sekce = '\0';

        if(*s != ENDSECTION) 
                return 0;

        return 1;
}


void break_line(char *line, char **pname, char **pvalue, char assignchar)
{
        *pname = NULL;
        *pvalue = NULL;

        if(line == NULL)
                return;

        if((*pvalue = strchr(line, assignchar)) == NULL)
                return;

        **pvalue = '\0';
        (*pvalue)++;

        *pname = line;

        return;
}
