/*
*  WhatsNew utility
*  
*  source/reporter.c
*  snapshot 20040207
*  
*  (c) 27.3.2002, 24.12.2003 Jaroslav Beran berk@volny.cz http://www.volny.cz/berk
*  License GNU/GPL2  
*/


#include <stdio.h>
#include <stdarg.h>
#ifdef __linux__
#include <unistd.h>
#else
#include <io.h>
#endif
#include <string.h>
#include <time.h>
#include <sys/stat.h>
#include "common.h"
#include "reporter.h"


char *rep_name = NULL;
char *tmp_name = NULL;
FILE *f = NULL;


int rep_init(char *rep_file, char *tmp_file)
{
        char *aktime;
        time_t cas;


        if ((rep_file == NULL) || (tmp_file == NULL))
                return -1;

        if (f != NULL)
                return -1;

        rep_name = rep_file;
        tmp_name = tmp_file;

        if ((f = fopen(tmp_name, "w")) == NULL)
                return -1;

        cas = time(NULL);
        aktime = ctime(&cas);
        aktime[strlen(aktime) - 1] = '\0';
        fprintf(f, "=== %s ===\n\n", aktime);

        return 0;
}


void rep_finish(void)
{
        char line[MAXREPLINE];
        FILE *old_f;


        if ((f == NULL) || (rep_name == NULL))
                return;

        if (my_touch(rep_name))
                return;

        if((old_f = fopen(rep_name, "r")) == NULL)
                return;

        while (fgets(line, MAXREPLINE, old_f) != NULL)
                fprintf(f, "%s", line);

        fclose(old_f);
        fclose(f);

        unlink(rep_name);
        rename(tmp_name, rep_name);

        rep_name = NULL;
        tmp_name = NULL;
        f = NULL;

        return;
}


void rep_writeln(const char *msgfmt, ...)
{
        va_list ap;
        char logmsg[MAXREPLINE] = "";


        if (f == NULL)
                return;

        va_start(ap, msgfmt);
        vsnprintf(logmsg, MAXREPLINE, msgfmt, ap);
        va_end(ap);

        fprintf(f, "%s\n", logmsg);

        return;
}


void rep_write(struct h_entry *entry, int rep_event)
{
        struct tm *t;


        if (f == NULL)
                return;

        if (entry == NULL)
                return;

        switch (rep_event){

                case REP_NEW:
                        fprintf(f, "(+");
                        break;

                case REP_DELETED:
                        fprintf(f, "(-");
                        break;

                case REP_MODIFIED:
                        fprintf(f, "(m");
                        break;

                case REP_UNREACHABLED:
                        fprintf(f, "(!");
                        break;

                default:
                        fprintf(f, "(?");
        }

        if (S_ISREG(entry->mode))
                fprintf(f, "f) ");

        if (S_ISDIR(entry->mode))
                fprintf(f, "d) ");

        #ifdef __linux__
        if (S_ISLNK(entry->mode))
                fprintf(f, "l) ");
        #endif

        t = localtime(&(entry->modified));
        fprintf(f, "%02d.%02d.%d %02d:%02d:%02d %s\n", \
                t->tm_mday, t->tm_mon + 1, \
                t->tm_year + 1900, t->tm_hour, \
                t->tm_min, t->tm_sec,
                entry->file);

        return;
}


FILE *rep_getfd(void)
{
        return f;
}
